"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PasswordConstraint = void 0;
const class_validator_1 = require("class-validator");
let PasswordConstraint = class PasswordConstraint {
    constructor() {
        this.defaultMessages = {
            blacklist: "Password is blacklisted",
            hasLowerCase: "Password must have at least one lowercase letter",
            hasNumbers: "Password must have at least one number",
            hasSpecialChars: "Password must have at least one special character",
            hasUpperCase: "Password must have at least one uppercase letter",
            minLength: "Password is too short",
            maxLength: "Password is too long",
            confirmPassword: "Passwords do not match",
        };
        this.defaultSpecialChars = [
            "!",
            "@",
            "#",
            "$",
            "%",
            "^",
            "&",
            "*",
            "(",
            ")",
        ];
        this.defaultBlacklistValues = ["12345678", "password"];
        this.errorMessage = "Password is invalid";
        this.defaultOptions = {
            blacklist: {
                enabled: true,
                values: this.defaultBlacklistValues,
                message: this.defaultMessages.blacklist,
            },
            hasLowerCase: {
                enabled: true,
                message: this.defaultMessages.hasLowerCase,
            },
            hasNumbers: {
                enabled: true,
                message: this.defaultMessages.hasNumbers,
            },
            hasSpecialChars: {
                enabled: true,
                message: this.defaultMessages.hasSpecialChars,
                characters: this.defaultSpecialChars,
            },
            hasUpperCase: {
                enabled: true,
                message: this.defaultMessages.hasUpperCase,
            },
            minLength: {
                enabled: true,
                length: 8,
                message: this.defaultMessages.minLength,
            },
            maxLength: {
                enabled: true,
                length: 15,
                message: this.defaultMessages.maxLength,
            },
            confirmPassword: {
                enabled: false,
                message: this.defaultMessages.confirmPassword,
                compareToProperty: "confirmPassword",
            },
        };
    }
    // Validate the password based on provided options or defaults
    validate(value, args) {
        var _a, _b, _c, _d, _e;
        const options = this.mergeWithDefaults((_a = args.constraints[0]) !== null && _a !== void 0 ? _a : {});
        const obj = args.object;
        if (((_b = options === null || options === void 0 ? void 0 : options.confirmPassword) === null || _b === void 0 ? void 0 : _b.enabled) &&
            !((_c = options === null || options === void 0 ? void 0 : options.confirmPassword) === null || _c === void 0 ? void 0 : _c.compareToProperty)) {
            throw Error("compareToProperty must be defined");
        }
        const confirmPassword = ((_d = options === null || options === void 0 ? void 0 : options.confirmPassword) === null || _d === void 0 ? void 0 : _d.enabled)
            ? obj[(_e = options === null || options === void 0 ? void 0 : options.confirmPassword) === null || _e === void 0 ? void 0 : _e.compareToProperty]
            : undefined;
        return this.validatePassword(value, confirmPassword, options);
    }
    // Return the default error message
    defaultMessage() {
        return this.errorMessage;
    }
    checkBlacklist(password, options) {
        var _a;
        if (options === null || options === void 0 ? void 0 : options.enabled) {
            const blacklist = (_a = options.values) !== null && _a !== void 0 ? _a : this.defaultBlacklistValues;
            if (blacklist.includes(password)) {
                this.errorMessage = options.message;
                return false;
            }
        }
        return true;
    }
    checkLength(password, options) {
        if ((options === null || options === void 0 ? void 0 : options.enabled) && password.length < options.length) {
            this.errorMessage = options.message;
            return false;
        }
        if ((options === null || options === void 0 ? void 0 : options.enabled) && password.length > options.length) {
            this.errorMessage = options.message;
            return false;
        }
        return true;
    }
    checkRegex(password, rule, regex) {
        if ((rule === null || rule === void 0 ? void 0 : rule.enabled) && !regex.test(password)) {
            this.errorMessage = rule.message;
            return false;
        }
        return true;
    }
    checkSpecialChars(password, options) {
        var _a;
        if (options === null || options === void 0 ? void 0 : options.enabled) {
            const chars = (_a = options === null || options === void 0 ? void 0 : options.characters) !== null && _a !== void 0 ? _a : this.defaultSpecialChars;
            const regex = new RegExp(`[${chars.join("")}]`, "g");
            if (!regex.test(password)) {
                this.errorMessage = options.message;
                return false;
            }
        }
        return true;
    }
    checkConfirmPassword(password, confirmPassword, options) {
        if ((options === null || options === void 0 ? void 0 : options.enabled) && confirmPassword !== password) {
            this.errorMessage = options.message;
            return false;
        }
        return true;
    }
    mergeWithDefaults(userOptions) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17;
        return {
            blacklist: {
                enabled: (_b = (_a = userOptions === null || userOptions === void 0 ? void 0 : userOptions.blacklist) === null || _a === void 0 ? void 0 : _a.enabled) !== null && _b !== void 0 ? _b : this.defaultOptions.blacklist.enabled,
                values: (_d = (_c = userOptions === null || userOptions === void 0 ? void 0 : userOptions.blacklist) === null || _c === void 0 ? void 0 : _c.values) !== null && _d !== void 0 ? _d : this.defaultOptions.blacklist.values,
                message: (_f = (_e = userOptions === null || userOptions === void 0 ? void 0 : userOptions.blacklist) === null || _e === void 0 ? void 0 : _e.message) !== null && _f !== void 0 ? _f : this.defaultMessages.blacklist,
            },
            hasLowerCase: {
                enabled: (_h = (_g = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasLowerCase) === null || _g === void 0 ? void 0 : _g.enabled) !== null && _h !== void 0 ? _h : this.defaultOptions.hasLowerCase.enabled,
                message: (_k = (_j = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasLowerCase) === null || _j === void 0 ? void 0 : _j.message) !== null && _k !== void 0 ? _k : this.defaultMessages.hasLowerCase,
            },
            hasNumbers: {
                enabled: (_m = (_l = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasNumbers) === null || _l === void 0 ? void 0 : _l.enabled) !== null && _m !== void 0 ? _m : this.defaultOptions.hasNumbers.enabled,
                message: (_p = (_o = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasNumbers) === null || _o === void 0 ? void 0 : _o.message) !== null && _p !== void 0 ? _p : this.defaultMessages.hasNumbers,
            },
            hasSpecialChars: {
                enabled: (_r = (_q = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasSpecialChars) === null || _q === void 0 ? void 0 : _q.enabled) !== null && _r !== void 0 ? _r : this.defaultOptions.hasSpecialChars.enabled,
                characters: (_t = (_s = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasSpecialChars) === null || _s === void 0 ? void 0 : _s.characters) !== null && _t !== void 0 ? _t : this.defaultOptions.hasSpecialChars.characters,
                message: (_v = (_u = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasSpecialChars) === null || _u === void 0 ? void 0 : _u.message) !== null && _v !== void 0 ? _v : this.defaultMessages.hasSpecialChars,
            },
            hasUpperCase: {
                enabled: (_x = (_w = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasUpperCase) === null || _w === void 0 ? void 0 : _w.enabled) !== null && _x !== void 0 ? _x : this.defaultOptions.hasUpperCase.enabled,
                message: (_z = (_y = userOptions === null || userOptions === void 0 ? void 0 : userOptions.hasUpperCase) === null || _y === void 0 ? void 0 : _y.message) !== null && _z !== void 0 ? _z : this.defaultMessages.hasUpperCase,
            },
            minLength: {
                enabled: (_1 = (_0 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.minLength) === null || _0 === void 0 ? void 0 : _0.enabled) !== null && _1 !== void 0 ? _1 : this.defaultOptions.minLength.enabled,
                length: (_3 = (_2 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.minLength) === null || _2 === void 0 ? void 0 : _2.length) !== null && _3 !== void 0 ? _3 : this.defaultOptions.minLength.length,
                message: (_5 = (_4 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.minLength) === null || _4 === void 0 ? void 0 : _4.message) !== null && _5 !== void 0 ? _5 : this.defaultMessages.minLength,
            },
            maxLength: {
                enabled: (_7 = (_6 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.maxLength) === null || _6 === void 0 ? void 0 : _6.enabled) !== null && _7 !== void 0 ? _7 : this.defaultOptions.maxLength.enabled,
                length: (_9 = (_8 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.maxLength) === null || _8 === void 0 ? void 0 : _8.length) !== null && _9 !== void 0 ? _9 : this.defaultOptions.maxLength.length,
                message: (_11 = (_10 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.maxLength) === null || _10 === void 0 ? void 0 : _10.message) !== null && _11 !== void 0 ? _11 : this.defaultMessages.maxLength,
            },
            confirmPassword: {
                enabled: (_13 = (_12 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.confirmPassword) === null || _12 === void 0 ? void 0 : _12.enabled) !== null && _13 !== void 0 ? _13 : this.defaultOptions.confirmPassword.enabled,
                message: (_15 = (_14 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.confirmPassword) === null || _14 === void 0 ? void 0 : _14.message) !== null && _15 !== void 0 ? _15 : this.defaultMessages.confirmPassword,
                compareToProperty: (_17 = (_16 = userOptions === null || userOptions === void 0 ? void 0 : userOptions.confirmPassword) === null || _16 === void 0 ? void 0 : _16.compareToProperty) !== null && _17 !== void 0 ? _17 : this.defaultOptions.confirmPassword.compareToProperty,
            },
        };
    }
    validatePassword(password, confirmPassword, options) {
        const checks = [
            this.checkBlacklist(password, options === null || options === void 0 ? void 0 : options.blacklist),
            this.checkLength(password, options === null || options === void 0 ? void 0 : options.minLength),
            this.checkRegex(password, options === null || options === void 0 ? void 0 : options.hasUpperCase, /[A-Z]/g),
            this.checkRegex(password, options === null || options === void 0 ? void 0 : options.hasLowerCase, /[a-z]/g),
            this.checkRegex(password, options === null || options === void 0 ? void 0 : options.hasNumbers, /\d/g),
            this.checkSpecialChars(password, options === null || options === void 0 ? void 0 : options.hasSpecialChars),
            this.checkConfirmPassword(password, confirmPassword, options === null || options === void 0 ? void 0 : options.confirmPassword),
        ];
        return checks.every((result) => result);
    }
};
exports.PasswordConstraint = PasswordConstraint;
exports.PasswordConstraint = PasswordConstraint = __decorate([
    (0, class_validator_1.ValidatorConstraint)({ async: true })
], PasswordConstraint);
//# sourceMappingURL=password.constraint.js.map